package com.winhc.service.impl;

import com.winhc.common.enums.CompanyEnum;
import com.winhc.config.ConfigConstant;
import com.winhc.kafka.KafkaProduce;
import com.winhc.service.RelationService;
import com.winhc.utils.CompanyUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.neo4j.driver.Driver;
import org.neo4j.driver.Session;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.winhc.utils.CompanyUtils.mergePerson;

/**
 * @author π
 * @Description:全量人员合并处理逻辑
 * @date 2021/1/11 10:03
 */
@Slf4j
@Service("personMergeAllImpl")
@AllArgsConstructor
public class PersonMergeAllImpl implements RelationService {

    @Autowired
    @Qualifier("DriverV1")
    Driver driver;
    @Autowired
    KafkaProduce kafkaProduce;
    @Autowired
    ConfigConstant configConstant;

    @Override
    public String save(List<Map<String, Object>> batch_list) {
        if (batch_list.isEmpty()) return null;
        long start = System.currentTimeMillis();
        Session session = driver.session();
        //1.查询关联人员
        final String query_cql =
                "\nWITH  {batch_list} AS batch_list \n" +
                        "UNWIND batch_list AS row \n" +
                        "MATCH (c1:" + CompanyEnum.Lable.COMPANY.code + "{company_id:row.first_company_id})-[r1]-(p1:" + CompanyEnum.Lable.PERSON.code + "{name:row.first_human_name})\n" +
                        "MATCH (p1)-[r2]-(c2:" + CompanyEnum.Lable.COMPANY.code + ")\n" +
                        ",(c3:" + CompanyEnum.Lable.COMPANY.code + " {company_id:row.second_company_id})-[r3]-(p3:" + CompanyEnum.Lable.PERSON.code + "{name:row.second_human_name})\n" +
                        "MATCH (p3)-[r4]-(c4:" + CompanyEnum.Lable.COMPANY.code + ")\n" +
                        "RETURN p1.person_id as new_human_pid, p3.person_id  as old_human_pid, count(distinct c2) as new_cnt, count(distinct c4) as old_cnt";
        log.info("query size: {}, cql:{}", batch_list.size(), query_cql);
        List<Map<String, String>> mergeData = CompanyUtils.writeNeo4j3(session, query_cql, new HashMap<String, Object>() {{
            put("batch_list", batch_list);
        }});

        //2.合并逻辑
        String data = mergePerson(session, mergeData);
        session.close();
        log.info("class:{} | save size:{} |  merge size:{} |cost:{}", PersonMergeAllImpl.class.getSimpleName(), batch_list.size(), mergeData.size(), (System.currentTimeMillis() - start));

        //3.发送变更人员记录
        if (StringUtils.isNotBlank(data)) {
            kafkaProduce.produce(configConstant.topic_pid_update_v1, data);
        }
        return data;
    }


}
