package com.winhc.service.impl;

import com.alibaba.fastjson.JSON;
import com.mongodb.BulkWriteError;
import com.mongodb.BulkWriteException;
import com.mongodb.bulk.BulkWriteResult;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.model.*;
import com.winhc.common.constant.Base;
import com.winhc.service.PidToMongoService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.bson.Document;
import org.bson.conversions.Bson;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

/**
 * @author π
 * @Description:案件-新增节点
 * @date 2021/5/17 10:03
 */
@Slf4j
@Service("pidToMongoServiceImpl")
@AllArgsConstructor
public class PidToMongoServiceImpl implements PidToMongoService {

    private final MongoTemplate mongoTemplate;


    @Override
    public Integer save(List<Map<String, Object>> ids) {
        BulkWriteResult bulkWriteResult = null;
        if (ids.size() > 0) {
            //todo 保存临时表,注意替换
            MongoCollection<Document> pid = mongoTemplate.getCollection(Base.PID_WAIT_MERGE_V9);
//            MongoCollection<Document> pid = mongoTemplate.getCollection("pid_wait_merge_v9_tmp");
            List<WriteModel<Document>> updateDocuments = new ArrayList<>();
            ids.forEach(m -> {
                Document filterDocument = new Document("_id", m.get("_id"));
                Document updateDocument = new Document("$set", new Document(m));
                UpdateOptions updateOptions = new UpdateOptions().upsert(true).bypassDocumentValidation(true);
                updateDocuments.add(new UpdateOneModel<>(filterDocument, updateDocument, updateOptions));
            });
            try {
                bulkWriteResult = pid.bulkWrite(updateDocuments, new BulkWriteOptions().ordered(false).bypassDocumentValidation(true));
            } catch (BulkWriteException e) {
                List<Map<String, Object>> collect = e.getWriteErrors().stream()
                        .map(b -> ids.get(b.getIndex()))
                        .collect(Collectors.toList());
                log.error("upset error records : {}", JSON.toJSONString(collect));
            }
        }
        return bulkWriteResult != null ? bulkWriteResult.getModifiedCount() : 0;
    }
}
